@@ -63,18 +63,13 @@ macro_rules! dbus_variant_sig_marshal {
     ($vname: ident, $($name: ident => $typ: path)+) => {
         impl $crate::Marshal for $vname {
             fn marshal(&self, ctx: &mut $crate::wire::marshal::MarshalContext) -> Result<(), $crate::Error> {
-                use $crate::Signature;
-
                 match self {
                     $(
                         Self::$name(v) => {
-                            let mut sig_str = String::new();
-                            <$typ as Signature>::signature().to_str(&mut sig_str);
-                            $crate::wire::marshal::base::marshal_base_param(
-                                &$crate::params::Base::Signature(sig_str),
-                                ctx,
-                            )
-                            .unwrap();
+                            let mut sig_str = $crate::wire::marshal::traits::SignatureBuffer::new();
+                            <$typ as $crate::Signature>::sig_str(&mut sig_str);
+                            let sig = $crate::wire::SignatureWrapper::new(sig_str.as_ref())?;
+                            sig.marshal(ctx)?;
                             v.marshal(ctx)?;
                         }
                     )+
@@ -388,12 +383,11 @@ fn test_variant_var_macro() {
 
     type StrRef<'buf> = &'buf str;
     // The point of the Path variant is to make sure types from other modules can be used. Do NOT change it to use a use-statement.
-    dbus_variant_var!(MyVariant, String => StrRef<'buf>; V2 => i32; Integer => u32; Path => crate::wire::marshal::traits::ObjectPath<&'buf str>);
+    dbus_variant_var!(MyVariant, String => StrRef<'buf>; V2 => i32; Integer => u32; Path => crate::wire::ObjectPath<&'buf str>);
     let v1 = MyVariant::String("ABCD");
     let v2 = MyVariant::V2(0);
     let v3 = MyVariant::Integer(100);
-    let object_path =
-        crate::wire::marshal::traits::ObjectPath::new("/org/freedesktop/DBus").unwrap();
+    let object_path = crate::wire::ObjectPath::new("/org/freedesktop/DBus").unwrap();
     let v4 = MyVariant::Path(object_path);
 
     (&v1, &v2, &v3, &v4).marshal(ctx).unwrap();
